@@ -22,6 +22,9 @@ Uses: ``os.path.isfile``.
 :throw: IOError
     '''
 
+    if type(paths) == str:
+        paths = [paths]
+
     for path in paths:
         if not os.path.isfile(path):
             raise IOError('"{0:s}" does not exist'.format(path))
@@ -36,6 +39,9 @@ Get the ``os.path.dirname`` of all file paths.
 :return: List of dirnames.
     '''
 
+    if type(paths) == str:
+        paths = [paths]
+
     ret = [os.path.dirname(path) for path in paths]
 
     if not return_unique:
@@ -53,9 +59,12 @@ List files that will be overwritten.
 :return: List of overwritten files.
     '''
 
+    if type(paths) == str:
+        paths = [paths]
+
     ret = [path for path in paths if os.path.isfile(path)]
 
-    if force:
+    if force or len(ret) == 0:
         return ret
 
     print('Files exist:')
@@ -293,8 +302,7 @@ Copy a file to a remote system. Uses ``scp -p``.
     _ExecCommand(cmd, verbose)
 
 
-def RsyncFromRemote(
-    hostname,
+def Rsync(
     source_dir,
     dest_dir,
     tempfilename,
@@ -303,7 +311,15 @@ def RsyncFromRemote(
     verbose=False,
     progress=True):
     r'''
-Copy files to a remote system using ``rsync -a --files-from``.
+Copy files to a destination using ``rsync -a --files-from``.
+
+:param str source_dir: Source directory.
+:param str dest_dir: Source directory.
+:param str tempfilename: Path of a temporary file to use to direct ``rsync --files-from``.
+:param list files: List of file-paths (relative to ``source_dir`` and ``dest_dir``).
+:param bool force: Continue without prompt.
+:param bool verbose: Verbose commands.
+:param bool progress: Show progress bar.
     '''
 
     assert type(tempfilename) == str
@@ -319,15 +335,15 @@ Copy files to a remote system using ``rsync -a --files-from``.
 
     if not progress:
 
-        cmd = 'rsync -a --files-from="{files:s}" {hostname:s}:{source_dir:s} {dest_dir:s}'.format(
-            hostname=hostname, source_dir=source_dir, dest_dir=dest_dir, files=tempfilename)
+        cmd = 'rsync -a --files-from="{files:s}" "{source_dir:s}" "{dest_dir:s}"'.format(
+            source_dir=source_dir, dest_dir=dest_dir, files=tempfilename)
 
         return _ExecCommand(cmd, verbose)
 
     # Run while printing output
 
-    cmd = 'rsync -aP --files-from="{files:s}" {hostname:s}:{source_dir:s} {dest_dir:s}'.format(
-        hostname=hostname, source_dir=source_dir, dest_dir=dest_dir, files=tempfilename)
+    cmd = 'rsync -aP --files-from="{files:s}" "{source_dir:s}" "{dest_dir:s}"'.format(
+        source_dir=source_dir, dest_dir=dest_dir, files=tempfilename)
 
     if verbose:
         print(cmd)
@@ -345,6 +361,38 @@ Copy files to a remote system using ``rsync -a --files-from``.
             sbar.update(e)
 
 
+def RsyncFromRemote(
+    hostname,
+    source_dir,
+    dest_dir,
+    tempfilename,
+    files,
+    force=False,
+    verbose=False,
+    progress=True):
+    r'''
+Copy files to a remote system using ``rsync -a --files-from``.
+
+:param str hostname: Hostname.
+:param str source_dir: Source directory.
+:param str dest_dir: Source directory.
+:param str tempfilename: Path of a temporary file to use to direct ``rsync --files-from``.
+:param list files: List of file-paths (relative to ``source_dir`` and ``dest_dir``).
+:param bool force: Continue without prompt.
+:param bool verbose: Verbose commands.
+:param bool progress: Show progress bar.
+    '''
+
+    return Rsync(
+        source_dir = hostname + ":" + source_dir,
+        dest_dir = dest_dir,
+        tempfilename = tempfilename,
+        files = files,
+        force = force,
+        verbose = verbose,
+        progress = progress)
+
+
 def RsyncToRemote(
     hostname,
     source_dir,
@@ -356,8 +404,52 @@ def RsyncToRemote(
     progress=True):
     r'''
 Copy files from a remote system using ``rsync -a --files-from``.
+
+:param str hostname: Hostname.
+:param str source_dir: Source directory.
+:param str dest_dir: Source directory.
+:param str tempfilename: Path of a temporary file to use to direct ``rsync --files-from``.
+:param list files: List of file-paths (relative to ``source_dir`` and ``dest_dir``).
+:param bool force: Continue without prompt.
+:param bool verbose: Verbose commands.
+:param bool progress: Show progress bar.
     '''
 
+    return Rsync(
+        source_dir = source_dir,
+        dest_dir = hostname + ":" + dest_dir,
+        tempfilename = tempfilename,
+        files = files,
+        force = force,
+        verbose = verbose,
+        progress = progress)
+
+
+def RsyncDiff(
+    source_dir,
+    dest_dir,
+    tempfilename,
+    files,
+    checksum=False,
+    force=False,
+    verbose=False):
+    r'''
+Check if files are different using *rsync*.
+*rsync* uses basic criteria such as file size and creation and modification date.
+This is much faster than using checksums but is only approximate.
+See `rsync manual <https://www.samba.org/ftp/rsync/rsync.html>`_.
+
+:param str source_dir: Source directory.
+:param str dest_dir: Source directory.
+:param str tempfilename: Path of a temporary file to use to direct ``rsync --files-from``.
+:param list files: List of file-paths (relative to ``source_dir`` and ``dest_dir``).
+:param bool checksum: Use checksum to test file difference.
+:param bool force: Continue without prompt.
+:param bool verbose: Verbose commands.
+    '''
+
+    files = [os.path.normpath(file) for file in files]
+
     assert type(tempfilename) == str
 
     if not force:
@@ -369,32 +461,59 @@ Copy files from a remote system using ``rsync -a --files-from``.
 
     # Run without printing output
 
-    if not progress:
+    opt = '-nai'
 
-        cmd = 'rsync -a --files-from="{files:s}" {source_dir:s} {hostname:s}:{dest_dir:s}'.format(
-            hostname=hostname, source_dir=source_dir, dest_dir=dest_dir, files=tempfilename)
+    if checksum:
+        opt += 'c'
 
-        return _ExecCommand(cmd, verbose)
+    cmd = 'rsync {opt:s} --files-from="{files:s}" "{source_dir:s}" "{dest_dir:s}"'.format(
+        source_dir=source_dir, dest_dir=dest_dir, files=tempfilename, opt=opt)
 
-    # Run while printing output
+    lines = list(filter(None, _ExecCommand(cmd, verbose).split('\n')))
+    lines = [line for line in lines if line[1] == 'f']
 
-    cmd = 'rsync -aP --files-from="{files:s}" {source_dir:s} {hostname:s}:{dest_dir:s}'.format(
-        hostname=hostname, source_dir=source_dir, dest_dir=dest_dir, files=tempfilename)
+    if len(lines) == 0:
+        return {
+            'skip' : np.ones((len(files)), dtype=np.bool),
+            'create' : np.zeros((len(files)), dtype=np.bool),
+            'overwrite' : np.zeros((len(files)), dtype=np.bool),
+        }
 
-    if verbose:
-        print(cmd)
+    check_paths = [line.split(' ')[1] for line in lines]
+    mode = np.zeros((len(check_paths)), dtype=np.int16)
 
-    pbar = tqdm.tqdm(total=len(files))
-    sbar = tqdm.tqdm(unit='B', unit_scale=True)
+    for i, line in enumerate(lines):
+        if line[0] == '>':
+            if line[2] == '+':
+                mode[i] = 1 # create
+            else:
+                mode[i] = 2 # overwrite
+        elif line[0] == '.':
+            pass
+        else:
+            raise IOError('Unknown cryptic output "{0:s}"'.format(line))
 
-    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
+    sorter = np.argsort(files)
+    source_paths = np.array(files, dtype=str)[sorter]
 
-    for line in iter(process.stdout.readline, ''):
-        line = line.decode("utf-8")
-        if re.match(r'(.*)(xfr\#)([0-9])(.*)(to\-chk\=)([0-9])(.*)', line):
-            e = int(list(filter(None, line.split(" ")))[-6].replace(",", ""))
-            pbar.update()
-            sbar.update(e)
+    i = np.argsort(check_paths)
+    check_paths = np.array(check_paths, dtype=str)[i]
+    mode = mode[i]
+
+    test = np.in1d(source_paths, check_paths)
+
+    idx = np.searchsorted(check_paths, source_paths)
+    idx = np.where(test, idx, 0)
+    ret = np.where(test, mode[idx], 0)
+    ret = ret.astype(np.int16)
+    out = np.empty_like(ret)
+    out[sorter] = ret
+
+    return {
+        'skip' : out == 0,
+        'create' : out == 1,
+        'overwrite' : out == 2,
+    }
 
 
 def MakeDir(dirname, force=False):
@@ -566,6 +685,7 @@ def ShelephantCopy(
     src_dir,
     dest_dir,
     checksum = False,
+    check_rsync = None,
     quiet = False,
     force = False,
     print_details = True,
@@ -588,6 +708,8 @@ Copy/move files.
 
 :param bool checksum: Use checksum to skip files that are the same.
 
+:param str check_rsync: Use rsync to check files to skip, specify temporary file name.
+
 :param bool quiet: Proceed without printing progress.
 
 :param bool force: Continue without prompt.
@@ -635,9 +757,23 @@ Copy/move files.
 
         create = [True for i in range(n)]
 
+    elif check_rsync is not None:
+
+        tmp = RsyncDiff(
+            source_dir = src_dir,
+            dest_dir = dest_dir,
+            tempfilename = check_rsync,
+            files = files,
+            checksum = checksum,
+            force = force)
+
+        skip = tmp['skip']
+        create = tmp['create']
+        overwrite = tmp['overwrite']
+
     else:
 
-        if checksum == True:
+        if checksum:
             src_checksums = GetChecksums(src, yaml_hostinfo_src, progress=not quiet)
             dest_checksums = GetChecksums(dest, yaml_hostinfo_dest, progress=not quiet)
 
@@ -731,6 +867,7 @@ def ShelephantCopySSH(
     src_dir,
     dest_dir,
     checksum = False,
+    check_rsync = None,
     quiet = False,
     force = False,
     print_details = True,
@@ -757,6 +894,8 @@ Get/send files.
 
 :param bool checksum: Use checksum to skip files that are the same.
 
+:param str check_rsync: Use rsync to check files to skip, specify temporary file name.
+
 :param bool quiet: Proceed without printing progress.
 
 :param bool force: Continue without prompt.
@@ -815,6 +954,20 @@ Get/send files.
 
         create = [True for i in range(n)]
 
+    elif check_rsync is not None:
+
+        tmp = RsyncDiff(
+            source_dir = src_dir if to_remote else hostname + ":" + src_dir,
+            dest_dir = hostname + ":" + dest_dir if to_remote else dest_dir,
+            tempfilename = check_rsync,
+            files = files,
+            checksum = checksum,
+            force = force)
+
+        skip = tmp['skip']
+        create = tmp['create']
+        overwrite = tmp['overwrite']
+
     else:
 
         if checksum == True:
